-- Clarity - By Dio (v1.0.5)
-- http://www.esoui.com/downloads/author-1518.html

local Clarity = {
	name = "Clarity",
	dbVersion = 2
}

ZO_CreateStringId("SI_BINDING_NAME_TOGGLE_CLARITY", "Toggle Clarity")

function ToggleClarity(active)
	Clarity.active = active ~= nil and active or not Clarity.active

	for _, addOn in ipairs(Clarity.tlw.addOns) do
		if addOn.enabled then
			Clarity:SetHidden(addOn)
		end
	end

	for _, other in ipairs(Clarity.tlw.other) do
		if other.enabled then
			Clarity:SetHidden(other)
		end
	end

	for _, zo in ipairs(Clarity.tlw.zo) do
		if zo.enabled then
			Clarity:SetHidden(zo)
		end
	end
end

function Clarity:Init()
	local defaults = {
		tlw = {
			toggleOnMenu = false,
			addOns = {},
			other = {},
			zo = {}
		}
	}

	self.db = ZO_SavedVars:New("Clarity_DB", self.dbVersion, nil, defaults)

	self:GetAddOns()
	self:FindTLW()
	self:MakeConfig()
	ToggleClarity(true)

	if self.db.toggleOnMenu then
		local prevActive = nil

		ZO_PreHookHandler(ZO_MainMenuCategoryBar, "OnShow", function()
			prevActive = self.active
			ToggleClarity(true)
		end)

		ZO_PreHookHandler(ZO_MainMenuCategoryBar, "OnHide", function()
			self.active = not prevActive
			ToggleClarity()
		end)
	end
end

function Clarity:GetAddOns()
	local AM = GetAddOnManager()
	local loadOutdatedAddOns = AM:GetLoadOutOfDateAddOns()
	self.addOns = {}

	for i = 1, AM:GetNumAddOns() do
		local name, title, _, _, enabled, _, outdated = AM:GetAddOnInfo(i)

		if enabled and (not outdated or loadOutdatedAddOns) then
			self.addOns[name] = title
		end
	end
end

function Clarity:SetHidden(obj)
	if not self.active then
		obj.tlw.SetHidden = obj.SetHidden
	end

	obj.tlw:SetHidden(self.active)

	if self.active then
		-- BWAHAHAHAHA
		obj.tlw.SetHidden = function() end
	end
end

function Clarity:MakeConfig()
	local panel = LibStub("EasyLAM-1.0"):MakePanel(self.name)

	local makeReload = function()
		panel:AddDescription("\nYou must reload the UI to apply changes!\n")

		panel:AddButton(
			"Reload UI",
			"",
			function()
				ReloadUI()
			end
		)
	end

	local sort = function(a, b)
		return a.title:lower() < b.title:lower()
	end

	panel:AddHeader("Options\n")

	panel:AddCheckbox(
		"Toggle Upon Menu",
		"Clarity will activate automatically upon character menus. Does not apply to system menus. After closing the menu, Clarity will return to its previous state.",
		function()
			return self.db.toggleOnMenu
		end,
		function(enable)
			self.db.toggleOnMenu = enable
		end,
		true,
		"Requires reloading the UI."
	)

	if #self.tlw.addOns > 0 then
		panel:AddHeader("Detected Add-Ons\n")
		table.sort(self.tlw.addOns, sort)

		for _, addOn in ipairs(self.tlw.addOns) do
			panel:AddCheckbox(
				addOn.title,
				"",
				function()
					return self.db.tlw.addOns[addOn.name]
				end,
				function(enable)
					self.db.tlw.addOns[addOn.name] = enable
					addOn.enabled = enable
				end
			)
		end

		makeReload()
	end

	if #self.tlw.other > 0 then
		panel:AddHeader("Other Frames\n")
		table.sort(self.tlw.other, sort)

		for _, other in ipairs(self.tlw.other) do
			panel:AddCheckbox(
				other.title,
				"",
				function()
					return self.db.tlw.other[other.title]
				end,
				function(enable)
					self.db.tlw.other[other.title] = enable
					other.enabled = enable
				end
			)
		end

		makeReload()
	end

	panel:AddHeader("Advanced - Core Frames\n")
	table.sort(self.tlw.zo, sort)

	for _, zo in ipairs(self.tlw.zo) do
		panel:AddCheckbox(
			zo.title,
			"",
			function()
				return self.db.tlw.zo[zo.title]
			end,
			function(enable)
				self.db.tlw.zo[zo.title] = enable
				zo.enabled = enable
			end
		)
	end

	makeReload()
end

function Clarity:FindTLW()
	local foundAddOns, dupeAddOns = {}, {}

	self.tlw = {
		zo = {},
		addOns = {},
		other = {}
	}

	for i = 1, GuiRoot:GetNumChildren() do
		local control = GuiRoot:GetChild(i)

		if control and control:GetType() == CT_TOPLEVELCONTROL then
			local controlName, found = control:GetName(), false
			local controlLower = controlName:lower()

			for addOnName, addOnTitle in pairs(self.addOns) do
				if controlLower:match(addOnName:lower()) then
					if foundAddOns[addOnName] then
						dupeAddOns[addOnName] = 1
					else
						foundAddOns[addOnName] = 1
					end

					table.insert(self.tlw.addOns, {
						name = addOnName,
						title = addOnTitle,
						tlw = control,
						SetHidden = control.SetHidden
					})

					found = true
					break
				end
			end

			if not found and not controlLower:match("tooltip") then
				if controlLower:match("^zo_") then
					table.insert(self.tlw.zo, {
						title = controlName,
						tlw = control,
						enabled = self.db.tlw.zo[controlName] or false,
						SetHidden = control.SetHidden
					})
				else
					table.insert(self.tlw.other, {
						title = controlName,
						tlw = control,
						enabled = self.db.tlw.other[controlName] or false,
						SetHidden = control.SetHidden
					})
				end
			end
		end
	end

	for _, addOn in ipairs(self.tlw.addOns) do
		if dupeAddOns[addOn.name] then
			addOn.name = addOn.tlw:GetName()
			addOn.title = addOn.name
		end

		addOn.enabled = self.db.tlw.addOns[addOn.name] or false
	end
end

EVENT_MANAGER:RegisterForEvent(Clarity.name, EVENT_PLAYER_ACTIVATED, function(event)
	EVENT_MANAGER:UnregisterForEvent(Clarity.name, event)
	Clarity:Init()
end)
